#!/bin/sh

PATH=/bin:/sbin:/usr/bin:/bin:/usr/sbin:/usr/local/bin
export PATH

MINSP="SP1"
SP="SP2"
NEWSYS=/data/newsys
ROOTFS=${NEWSYS}/rootfs
HEADER="Installing ${SP}"

lng=$1
case $lng in
   local) ;;
   de) ;;
   en) ;;
   *) lng="de" ;;
esac

update() {
   rm -f /tmp/REBOOT

   if [ ! -x /usr/bin/opkg -a -x /usr/bin/opkg-cl ]; then
      ln /usr/bin/opkg-cl /usr/bin/opkg
   fi

   if [ ! -x /usr/bin/opkg ]; then
      echo "================================================================="
      echo "===== es fehlt das Programm 'opkg' fr den Updateprozess! ======="
      echo "================================================================="

      case $lng in
	 de) lcdmsg -m "OPKG ist nicht\ninstalliert!" ;;
	 en) lcdmsg -m "OPKG is not\ninstalled!" ;;
      esac
      sleep 3

      exit 1
   fi

   if [ -x /usr/bin/ecc_update ]; then
      /usr/bin/ecc_update START
   fi

   # laufende Prozesse beenden
   if [ -f /var/run/eccweb.pid ]; then
      /sbin/start-stop-daemon -K -q -p /var/run/eccweb.pid
   fi
   if [ -f /var/run/ecc_core.pid ]; then
      /sbin/start-stop-daemon -K -q -p /var/run/ecc_core.pid
   fi
   if [ -f /var/run/ecc_modbus.pid ]; then
      /sbin/start-stop-daemon -K -q -p /var/run/ecc_modbus.pid
   fi
   if [ -f /var/run/ecc_bacnet.pid ]; then
      /sbin/start-stop-daemon -K -q -p /var/run/ecc_bacnet.pid
   fi
   if [ -f /var/run/ecc_knx.pid ]; then
      /sbin/start-stop-daemon -K -q -p /var/run/ecc_knx.pid
   fi

   /usr/bin/lcdmsg -H "${HEADER}" -m "unpacking files ..."

   mkdir -p ${NEWSYS}

   /bin/gunzip < package.gz | /bin/tar --no-same-owner -xvf - -C ${NEWSYS}

   /usr/bin/lcdmsg -H "${HEADER}" -m "extracting new\nroot FS ..."
   /bin/tar -xvf ${NEWSYS}/rootfs.tar -C ${NEWSYS}
   rm ${NEWSYS}/rootfs.tar

   # save some of the configuration files to the new system
   mkdir ${NEWSYS}/etc/ssh
   mkdir ${NEWSYS}/data
   cp /etc/ssh_host* ${NEWSYS}/etc/ssh
   cp /etc/network/interfaces ${NEWSYS}/etc/network
   cp /etc/resolv.conf ${NEWSYS}/etc
   cp /root/delivery.log ${NEWSYS}/root
   cp /root/wdog.log ${NEWSYS}/root
   cp /root/.bash_history ${NEWSYS}/root
   cp /root/.exrc ${NEWSYS}/root
   cp /root/clean_keys.sh ${NEWSYS}/root

   mkdir -p ${ROOTFS}
   mount --bind / ${ROOTFS}
   mount --bind /data ${NEWSYS}/data
   mount --bind /sys ${NEWSYS}/sys
   mount --bind /proc ${NEWSYS}/proc
   mount --bind /tmp ${NEWSYS}/tmp

   /etc/init.d/S10udev stop

   sync
   /usr/sbin/chroot ${NEWSYS} /copyfs.sh

   case $lng in
      de) lcdmsg -H "${HEADER}" -m "Abbruch der\nInstallation" ;;
      en) lcdmsg -H "${HEADER}" -m "Aborting installation" ;;
   esac
   /bin/sync

   touch ${NEWSYS}/etc/${SP}
   touch /tmp/REBOOT
   case $lng in
      de) lcdmsg -H "${HEADER}" -m "fertig... Neustart\nnach Stick abziehen" ;;
      en) lcdmsg -H "${HEADER}" -m "ready... restart\nafter removing Stick" ;;
   esac
}

rc=0
echo "Running Update Check"

if [ -x /usr/bin/lcdmsg ]; then

   killall lcd_manager

   if [ ! -f /etc/${MINSP} ]; then
      if [ "$lng" = "local" ]; then
         echo "${MINSP} needed but NOT installed ... give up"
	 exit 0
      else
	 case $lng in
	    de) lcdmsg -m "Service-Pack ${MINSP}\nwird bentigt" -c "Abbruch"
		;;
	    en) lcdmsg -m "Service-Pack ${MINSP}\nis needed" -c "Cancel"
		;;
	 esac
	 exit 0
      fi
   fi

   if [ -f /etc/${SP} ]; then
      if [ "$lng" = "local" ]; then
         echo "${SP} already installed ... give up"
	 exit 0
      else
	 case $lng in
	    de) lcdmsg -m "Service-Pack ${SP}\nist bereits\ninstalliert" -c "Abbruch"
		;;
	    en) lcdmsg -m "Service-Pack ${SP}\nalready installed" -c "Cancel"
		;;
	 esac
	 exit 0
      fi
   fi

   if [ "$lng" = "local" ]; then
      rc=0
   else
      case $lng in
	 de) lcdmsg -m "Mchten Sie das\nService-Pack ${SP}\ninstallieren?" -c "Abbruch" -o "UPDATE"
	     rc=$?
	     ;;
	 en) lcdmsg -m "Would you like to\ninstall the\nService-Pack ${SP}?" -c "Cancel" -o "UPDATE"
	     rc=$?
	     ;;
      esac
      echo "Selection: $rc"
   fi

   case $rc in
      0) case $lng in
            de) lcdmsg -m "${SP} wird installiert" ;;
            en) lcdmsg -m "${SP} is installing" ;;
	 esac
	 update
	 ;;
      *) case $lng in
      	    de) lcdmsg -m "Abgebrochen ...\nbitte Stick abziehen" ;;
      	    en) lcdmsg -m "Aborted ...\nplease remove Stick" ;;
	 esac
	 rc=2
	 ;;
   esac
else

   echo "================================================================="
   echo "==== es fehlt das Programm 'lcdmsg' fr den Updateprozess! ======"
   echo "================================================================="

   exit 1

fi

sync
echo "... FINISHED ..."

exit $rc
